/**
*****************************************************************************
**
**  File        : main.c
**  Abstract    : main function.
**
**
**
*****************************************************************************
*/


/* Includes */
#include <stddef.h>
#include "stm32f10x.h"
#include "lib_pcd8544.h"
#include "defines.h"

/* Private typedef */
/* Private define  */

/* Private macro */
/* Private variables */
int tmp;
uint16_t licznik = 0;

//czestotliwosc w 10^2 Hz
uint16_t FreqRed = 0;
uint16_t FreqGreen = 0;
uint16_t FreqBlue = 0;
uint16_t FreqClear = 0;

RCC_ClocksTypeDef RCC_Clocks;
TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;

/* Private function prototypes */
/* Private functions */
void Configuration_RCC(void);
void Configuration_Interrupts(void);
void Configuration_GPIO(void);

/**
**===========================================================================
**
**  Abstract: main program
**
**===========================================================================
*/
int main(void)
{
	//konfiguracja
	Configuration_RCC();
	Configuration_Interrupts();
	Configuration_GPIO();

	//ustawienie nEO
	TCS3200_DISABLE;
	//wybranie filtru - brak
	TCS3200_Clear;
	//ustawienie skali czstotliwosci
	TCS3200_Freq_20p;
	//wlaczenie diod LED
	TCS3200_LED_ON;

	//konfiguracja licznika
	TIM_TimeBaseStructure.TIM_Period = 999;//9999;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;//1000
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

	//konfiguracja sygnau
	TIM_TIxExternalClockConfig(TIM2, TIM_TIxExternalCLK1Source_TI1, TIM_ICPolarity_Rising, 0x0);

	//wlaczenie licznika
	TIM_Cmd(TIM2, ENABLE);

	//pobranie informacji o czstotliwosci
	RCC_GetClocksFreq(&RCC_Clocks);

	//konfiguracja SysTick
	SysTick_Config(RCC_Clocks.SYSCLK_Frequency/100);

	//inicjalizacja wuwietlacza moduu KAmodLCD1
	PCD8544_INIT();
	PCD8544_LCDClear();

	//aktualizacja informacji wyswietlanych na wyswietlaczu
	PCD8544_LCDWriteText("KAmodColor",12,0);
	PCD8544_LCDWriteText("Clear 00,0 kHz",0,1);		//14
	PCD8544_LCDWriteText("Red 00,0 kHz",6,2);		//12
	PCD8544_LCDWriteText("Green 00,0 kHz",0,3);		//14
	PCD8544_LCDWriteText("Blue 00,0 kHz",3,4);		//13
	PCD8544_LCDWrite();

	//nieskoczona ptla
	while (1)
	{
		//aktualizacja informacji, filtr - Clear
		tmp = FreqClear;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,60-6*1,1);
		tmp = tmp / 10;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,60-6*3,1);
		tmp = tmp / 10;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,60-6*4,1);

		//aktualizacja informacji, filtr - Red
		tmp = FreqRed;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,54-6*1,2);
		tmp = tmp / 10;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,54-6*3,2);
		tmp = tmp / 10;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,54-6*4,2);

		//aktualizacja informacji, filtr - Green
		tmp = FreqGreen;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,60-6*1,3);
		tmp = tmp / 10;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,60-6*3,3);
		tmp = tmp / 10;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,60-6*4,3);

		//aktualizacja informacji, filtr - Blue
		tmp = FreqBlue;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,3+54-6*1,4);
		tmp = tmp / 10;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,3+54-6*3,4);
		tmp = tmp / 10;
		PCD8544_LCDWriteSign((tmp % 10) + 0x30,3+54-6*4,4);

		//aktualizacja danych wywietlanych na LCD
		PCD8544_LCDWrite();
	}
}

void Configuration_RCC(void)
{
	ErrorStatus HSEStartUpStatus;

	//Reset ustawien RCC
	RCC_DeInit();

	//Wlaczaenie HSE
	RCC_HSEConfig(RCC_HSE_ON);

	//Zaczekaj az HSE zostanie zainicjowany
	HSEStartUpStatus = RCC_WaitForHSEStartUp();

	//jesli HSE zosta zainicjalizowany poprawnie
	if(HSEStartUpStatus == SUCCESS)
	{
		//Opoznienie dla pamieci Flash
		FLASH_SetLatency(FLASH_Latency_2);

		//Wlaczenie buffora pamieci flash
		FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

		//konfiguracja preskalera AHB
		//HCLK = SYSCLK
		RCC_HCLKConfig(RCC_SYSCLK_Div1);

		//konfigracja preskalera APB2
	    //PCLK2 = HCLK = 72MHz, czestotliwosz 72MHz bedzie uzyskana dopiero po pelnej inicjalizacji RCC
		RCC_PCLK2Config(RCC_HCLK_Div1);

	    //konfigracja preskalera APB1
		//PCLK1 = HCLK/2 = 36MHz, czestotliwosz 36MHz bedzie uzyskana dopiero po pelnej inicjalizacji RCC
		RCC_PCLK1Config(RCC_HCLK_Div2);

	    //konfiguracja PLLLCK
		//PLLCLK = HSE / 1 * 9 = 72 MHz
		RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

		//Enable PLL
	    RCC_PLLCmd(ENABLE);

	    //odczekanie az PLL bedzie gotowy do pracy
	    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET){}

	    //konfiguracja SW
		//wybranie PLL jako SYSCLK
		RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

	    //odczekanie az PLLCLK bedzie sygnaem zegarowym mikrokontrolera
	    while((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != (uint32_t)0x08){}
	}

	//wczenie zegarw wykorzystywanych peryferii APB1
	RCC_APB1PeriphClockCmd(
		RCC_APB1Periph_TIM2 |
		RCC_APB1Periph_TIM3
		, ENABLE);

	//wczenie zegarw wykorzystywanych peryferii APB2
	RCC_APB2PeriphClockCmd(
		RCC_APB2Periph_TCS3200 |
		RCC_APB2Periph_AFIO |
		RCC_APB2Periph_GPIOB
		, ENABLE);
}

void Configuration_Interrupts(void)
{
	//zdefiniowanie struktury wykorzystywanej do konfiguracji przerwani
	NVIC_InitTypeDef NVIC_InitStructure;

	// W przeciwnym wypadku ustaw na 0x08000000
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
}

void Configuration_GPIO(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	//inicjalizacja GPIO - S0, S1, S2, S3, nEO, LED
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin =	GPIO_Pin(TCS3200_PIN_S0) |
									GPIO_Pin(TCS3200_PIN_S1) |
									GPIO_Pin(TCS3200_PIN_S2) |
									GPIO_Pin(TCS3200_PIN_S3) |
									GPIO_Pin(TCS3200_PIN_nOE) |
									GPIO_Pin(TCS3200_PIN_LED);
	GPIO_Init(GPIO_TCS3200, &GPIO_InitStructure);

	//inicjalizacja GPIO - OUT
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin = TCS3200_PIN_OUT;
	GPIO_Init(GPIO_TCS3200, &GPIO_InitStructure);

}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/*
 * Minimal __assert_func used by the assert() macro
 * */
void __assert_func(const char *file, int line, const char *func, const char *failedexpr)
{
  while(1)
  {}
}

/*
 * Minimal __assert() uses __assert__func()
 * */
void __assert(const char *file, int line, const char *failedexpr)
{
   __assert_func (file, line, NULL, failedexpr);
}

#ifdef USE_SEE
#ifndef USE_DEFAULT_TIMEOUT_CALLBACK
/**
  * @brief  Basic management of the timeout situation.
  * @param  None.
  * @retval sEE_FAIL.
  */
uint32_t sEE_TIMEOUT_UserCallback(void)
{
  /* Return with error code */
  return sEE_FAIL;
}
#endif
#endif /* USE_SEE */

